<?php

namespace GopNext\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class CalculationBudgetController {
    
    protected $crudService;
    protected $service;
  
    public function __construct($connection, $environment, $util, $crudService, $service) {
        $this->connection   = $connection;
        $this->environment  = $environment;
        $this->util         = $util;
        $this->crudService	= $crudService;
        $this->service      = $service;
    }
    
    public function budgetParameters(Request\Filter $request, Response $response) {
        $params = ['CDOPERADOR', 'NRORG'];
        $conditions = $this->util->prepareConditions($params);
        
        $data = $this->crudService->getFilteredDataSet('budgetParameters', $conditions)->getRows();
		$response->addDataSet(new DataSet("budgetParameters", $data));
    }
    
    public function gridSummary(Request\Filter $request, Response $response) {
        $params = ['CDOPERADOR', 'NRORG'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getFilterCriteria()->getConditions();
        
        foreach ($filter as &$item) {
            if ($item["columnName"] === "NRPERIODOORCA" && is_array($item["value"])) {
                $item["value"] = $item["value"]["NRPERIODOORCA"];
            }
        }
        
        $data = $this->crudService->getFilteredDataSet('gridSummary', array_merge($conditions, $filter))->getRows();
		$response->addDataSet(new DataSet("gridSummary", $data));
    }
    
    public function gridMainGrid(Request\Filter $request, Response $response) {
        $params = ['CDOPERADOR', 'NRORG'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getFilterCriteria()->getConditions();
        
        foreach ($filter as &$item) {
            if ($item["columnName"] === "NRPERIODOORCA" && is_array($item["value"])) {
                $item["value"] = $item["value"]["NRPERIODOORCA"];
            }
        }
        $data = $this->crudService->getFilteredDataSet('gop10000_mainGrid', array_merge($conditions, $filter))->getRows();
		$response->addDataSet(new DataSet("gop10000_mainGrid", $data));
    }
    
    public function changeBudgetMonth(Request\DataSet $request, Response $response) {
        $data = $request->getParameters();
        try {
            $this->service->beginTransaction();
            foreach ($data["dataset"] as $row) {
                $params = [
                    "NRORCAMENTOGOP" => $row["NRORCAMENTOGOP"],
                    "IDEDITMES"      => $row["IDEDITMES"],
                    "JANEIRO"        => $row["JANEIRO"],
                    "FEVEREIRO"      => $row["FEVEREIRO"],
                    "MARCO"          => $row["MARCO"],
                    "ABRIL"          => $row["ABRIL"],
                    "MAIO"           => $row["MAIO"],
                    "JUNHO"          => $row["JUNHO"],
                    "JULHO"          => $row["JULHO"],
                    "AGOSTO"         => $row["AGOSTO"],
                    "SETEMBRO"       => $row["SETEMBRO"],
                    "OUTUBRO"        => $row["OUTUBRO"],
                    "NOVEMBRO"       => $row["NOVEMBRO"],
                    "DEZEMBRO"       => $row["DEZEMBRO"],
                    "NRORG"          => $row["NRORG"]
                ];
                $this->service->changeBudgetMonth($params);
            }
          $this->service->commit();  
        }catch(\Exception $ex) {
          $this->service->rollback();
          $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function budgetCalculation(Request\DataSet $request, Response $response) {
        $data = $request->getParameters();
        
        try {
            $this->service->beginTransaction();
            $nrLista = $this->util->insertLista($data['CDFILIAL']);
       		$this->service->budgetCalculation($data, $nrLista);
			$this->util->deleteLista($nrLista);
			
            $this->service->commit();
        } catch(\Exception $ex) {
            $this->service->rollback();
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function completeBudget(Request\DataSet $request, Response $response) {
        $data = $request->getParameters();
        
        try {
            $this->service->beginTransaction();
            $nrLista = $this->util->insertLista($data['CDFILIAL']);
       		$this->service->completeBudget($data, $nrLista);
			$this->util->deleteLista($nrLista);
			
            $this->service->commit();
        } catch(\Exception $ex) {
            $this->service->rollback();
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function workingDays(Request\Row $request, Response $response) {
        $parameters = $request->getRow();
        $changedValue = $parameters['EDITEDVALUE'];
        $params = ['NRORG'];
        $conditions = $this->util->prepareConditions($params);
        $dailyValue = 0;
        
        array_push($conditions, 
            array(
                "columnName" => "CDFILIAL",
                "operator"   => '=',
                "value"      => $parameters['CDFILIAL']
            ),
            array(
                "columnName" => "MES",
                "operator"   => '=',
                "value"      => $parameters['MES']
            ),
            array(
                "columnName" => "ANO",
                "operator"   => '=',
                "value"      => $parameters['ANO']
            )
        );
        $data = $this->crudService->getFilteredDataSet('workingDays', $conditions)->getRows();
        
        foreach ($data as $item) {
            if (intval($parameters['MES']) === intval(substr($item['MES'], -2))) {
                $dailyValue = $changedValue / $item['NRDIASUTEIS'];
            } else {
                $dataset['dataset'][] = array(
                    'MES' => intval(substr($item['MES'], -2)),
                    'VALOR' => $item['NRDIASUTEIS'] * $dailyValue,
                );
            }
        }
        
		$response->addDataSet(new DataSet("workingDays", $dataset));
    }
}